package com.ruoyi.project.mapper;

import com.ruoyi.project.bo.MyMonthDays;
import com.ruoyi.project.bo.ProjectStaffRecordInput;
import com.ruoyi.project.bo.ProjectStaffRecordOut;
import com.ruoyi.project.domain.ProjectStaffRecord;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface ProjectStaffRecordMapper {
    int deleteByPrimaryKey(Long id);

    int insert(ProjectStaffRecord record);

    int insertSelective(ProjectStaffRecord record);

    ProjectStaffRecord selectByPrimaryKey(Long id);

    int updateByPrimaryKeySelective(ProjectStaffRecord record);

    int updateByPrimaryKey(ProjectStaffRecord record);

    List<ProjectStaffRecordOut> selectList(ProjectStaffRecordInput in);

    int selectById(Long id);


    List<ProjectStaffRecord> selectByPidAndAid(@Param("pid") Long pid, @Param("aid") Long aid);

    List<MyMonthDays> myDurationMonth(Long id, Long aid, String lastDayOfMonth, String s);

    ProjectStaffRecordOut selectDetailsById(Long id);
}
